<?php

namespace SGalinski\SgMail\Updates;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgMail\Service\BackendService;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Install\Updates\AbstractUpdate;

/**
 * Fix all incorrect / deprecated language codes (be languages instead of sys languages) in the database
 */
class UpdateLanguages extends AbstractUpdate {
	/**
	 * @var string
	 */
	protected $title = 'Find all templates & queue entries with an incorrect (be languages instead of sys languages) language code and fix it';

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManager
	 */
	protected $objectManager;

	/**
	 * @var array
	 */
	protected $tables = [
		'tx_sgmail_domain_model_mail', 'tx_sgmail_domain_model_template'
	];

	/**
	 * is used to map language codes. If empty the updatewizard will ignore this update
	 * example: you want to change en-us to en
	 * [
	 *    'en-us' => 'en'
	 * ]
	 *
	 * @var array
	 */
	protected $languageMap = [

	];

	/**
	 * Checks whether updates are required.
	 *
	 * @param string &$description The description for the update
	 * @return bool Whether an update is required (TRUE) or not (FALSE)
	 * @throws \InvalidArgumentException
	 */
	public function checkForUpdate(&$description) {
		$upgradeNecessary = FALSE;

		if (count($this->languageMap) > 0) {
			$languages = BackendService::getLanguages();
			$where = 'language NOT IN (';
			foreach ($languages as $language) {
				$where .= '"' . $language['isocode'] . '",';
			}
			$where = rtrim($where, ',');
			$where .= ')';

			$description = 'Check all the language codes in the database';

			/** @var DatabaseConnection $databaseConnection */
			$databaseConnection = $this->getDatabaseConnection();

			foreach ($this->tables as $table) {
				/** @var \mysqli_result $result */
				$result = $databaseConnection->exec_SELECTquery('distinct language', $table, $where);
				if ($result->num_rows > 0) {
					$upgradeNecessary = TRUE;
					break;
				}
			}
		}

		return !(!$upgradeNecessary || $this->isWizardDone());
	}

	/**
	 * Performs the accordant updates.
	 *
	 * @param array &$dbQueries Queries done in this update
	 * @param mixed &$customMessages Custom messages
	 * @return bool Whether everything went smoothly or not
	 */
	public function performUpdate(array &$dbQueries, &$customMessages) {
		$dbQueries = [];

		/** @var DatabaseConnection $databaseConnection */
		$databaseConnection = $this->getDatabaseConnection();

		foreach ($this->tables as $table) {
			foreach ($this->languageMap as $origin => $target) {
				$where = 'language = ' . $origin;
				$databaseConnection->exec_UPDATEquery($table, $where, [
					'language' => $target
				]);
				$dbQueries[] = $databaseConnection->debug_lastBuiltQuery;
			}
		}

		$this->markWizardAsDone();
		return TRUE;
	}
}
